@@ -1,18 +1,22 @@
-__version__ = "0.15.0"
+__version__ = "0.16.0"
 
 from e3nn_jax._src.config import config
 from e3nn_jax._src.rotation import (
     rand_matrix,
+    rotation_angle_from_matrix,
     identity_angles,
     rand_angles,
     compose_angles,
     inverse_angles,
+    rotation_angle_from_angles,
     identity_quaternion,
     rand_quaternion,
     compose_quaternion,
     inverse_quaternion,
+    rotation_angle_from_quaternion,
     rand_axis_angle,
     compose_axis_angle,
+    rotation_angle_from_axis_angle,
     matrix_x,
     matrix_y,
     matrix_z,
@@ -32,6 +36,7 @@ from e3nn_jax._src.rotation import (
     identity_log_coordinates,
     compose_log_coordinates,
     inverse_log_coordinates,
+    rotation_angle_from_log_coordinates,
     log_coordinates_to_matrix,
     matrix_to_log_coordinates,
     log_coordinates_to_quaternion,
@@ -44,7 +49,7 @@ from e3nn_jax._src.rotation import (
     xyz_to_angles,
 )
 from e3nn_jax._src.su2 import su2_clebsch_gordan, su2_generators
-from e3nn_jax._src.so3 import clebsch_gordan, wigner_D, generators
+from e3nn_jax._src.so3 import clebsch_gordan, generators
 from e3nn_jax._src.instruction import Instruction
 from e3nn_jax._src.irreps import Irrep, MulIrrep, Irreps
 from e3nn_jax._src.irreps_array import IrrepsArray, concatenate, stack, mean, norm, normal
@@ -52,7 +57,6 @@ from e3nn_jax._src.irreps_array import sum_ as sum
 from e3nn_jax._src.spherical_harmonics import spherical_harmonics, sh, legendre
 from e3nn_jax._src.radial import sus, soft_one_hot_linspace, bessel, poly_envelope, soft_envelope
 from e3nn_jax._src.linear import FunctionalLinear
-from e3nn_jax._src.linear_haiku import Linear  # TODO remove
 from e3nn_jax._src.core_tensor_product import FunctionalTensorProduct
 from e3nn_jax._src.tensor_products import (
     FunctionalFullyConnectedTensorProduct,
@@ -63,18 +67,13 @@ from e3nn_jax._src.tensor_products import (
 from e3nn_jax._src.grad import grad
 from e3nn_jax._src.activation import scalar_activation, normalize_function
 from e3nn_jax._src.gate import gate
-from e3nn_jax._src.batchnorm import BatchNorm  # TODO remove
-from e3nn_jax._src.dropout import Dropout  # TODO remove
-from e3nn_jax._src.mlp_haiku import MultiLayerPerceptron  # TODO remove
 from e3nn_jax._src.graph_util import index_add, radius_graph, scatter_sum
 from e3nn_jax._src.reduced_tensor_product import (
     reduced_tensor_product_basis,
     reduced_symmetric_tensor_product_basis,
     reduced_antisymmetric_tensor_product_basis,
 )
-from e3nn_jax._src.symmetric_tensor_product import SymmetricTensorProduct  # TODO remove
-from e3nn_jax._src.s2grid import from_s2grid, to_s2grid, s2_irreps, s2grid, s2grid_vectors, pad_to_plot_on_s2grid
-from e3nn_jax._src.fc_tp_haiku import FullyConnectedTensorProduct  # TODO remove
+from e3nn_jax._src.s2grid import s2_irreps, to_s2grid, to_s2point, from_s2grid, s2_sum_of_diracs, SphericalSignal
 
 # make submodules flax and haiku available
 from e3nn_jax import flax, haiku
@@ -82,16 +81,20 @@ from e3nn_jax import flax, haiku
 __all__ = [
     "config",  # not in docs
     "rand_matrix",
+    "rotation_angle_from_matrix",
     "identity_angles",
     "rand_angles",
     "compose_angles",
     "inverse_angles",
+    "rotation_angle_from_angles",
     "identity_quaternion",
     "rand_quaternion",
     "compose_quaternion",
     "inverse_quaternion",
+    "rotation_angle_from_quaternion",
     "rand_axis_angle",
     "compose_axis_angle",
+    "rotation_angle_from_axis_angle",
     "matrix_x",
     "matrix_y",
     "matrix_z",
@@ -111,6 +114,7 @@ __all__ = [
     "identity_log_coordinates",
     "compose_log_coordinates",
     "inverse_log_coordinates",
+    "rotation_angle_from_log_coordinates",
     "log_coordinates_to_matrix",
     "matrix_to_log_coordinates",
     "log_coordinates_to_quaternion",
@@ -124,7 +128,6 @@ __all__ = [
     "su2_clebsch_gordan",  # not in docs
     "su2_generators",  # not in docs
     "clebsch_gordan",
-    "wigner_D",  # TODO could be moved into Irrep
     "generators",  # TODO could be moved into Irrep
     "Instruction",  # not in docs
     "Irrep",
@@ -144,10 +147,8 @@ __all__ = [
     "soft_one_hot_linspace",
     "bessel",
     "FunctionalLinear",  # not in docs
-    "Linear",
     "FunctionalTensorProduct",
     "FunctionalFullyConnectedTensorProduct",  # deprecated
-    "FullyConnectedTensorProduct",  # deprecated
     "tensor_product",
     "elementwise_tensor_product",
     "tensor_square",
@@ -155,9 +156,6 @@ __all__ = [
     "scalar_activation",
     "normalize_function",
     "gate",
-    "BatchNorm",
-    "Dropout",
-    "MultiLayerPerceptron",
     "index_add",
     "radius_graph",
     "scatter_sum",
@@ -166,13 +164,12 @@ __all__ = [
     "reduced_tensor_product_basis",
     "reduced_symmetric_tensor_product_basis",
     "reduced_antisymmetric_tensor_product_basis",
-    "SymmetricTensorProduct",
-    "from_s2grid",
-    "to_s2grid",
     "s2_irreps",
-    "s2grid",
-    "s2grid_vectors",
-    "pad_to_plot_on_s2grid",
+    "to_s2grid",
+    "to_s2point",
+    "from_s2grid",
+    "s2_sum_of_diracs",
+    "SphericalSignal",
     "flax",
     "haiku",
 ]
