@@ -1,9 +1,10 @@
+from math import prod
 from typing import Callable, List
 
-import e3nn_jax as e3nn
 import jax
 import jax.numpy as jnp
-from e3nn_jax._src.util.prod import prod
+
+import e3nn_jax as e3nn
 
 
 def grad(
