@@ -10,7 +10,7 @@ import jax.scipy
 import numpy as np
 
 import e3nn_jax as e3nn
-from e3nn_jax import Irreps, axis_angle_to_angles, config, matrix_to_angles, quaternion_to_angles
+from e3nn_jax import Irreps
 from e3nn_jax._src.irreps import IntoIrreps
 
 
@@ -152,6 +152,27 @@ class IrrepsArray:
             array = jnp.zeros(leading_shape + (0,), dtype)
         return IrrepsArray(irreps=irreps, array=array, list=list)
 
+    @staticmethod
+    def as_irreps_array(array: Union[jnp.ndarray, "IrrepsArray"], *, backend=None):
+        """Convert an array to an IrrepsArray.
+
+        Args:
+            array (jax.numpy.ndarray or IrrepsArray): array to convert
+
+        Returns:
+            IrrepsArray
+        """
+        if isinstance(array, IrrepsArray):
+            return array
+
+        jnp = _infer_backend(array) if backend is None else backend
+        array = jnp.asarray(array)
+
+        if array.ndim == 0:
+            raise ValueError("IrrepsArray.as_irreps_array: Cannot convert an array of rank 0 to an IrrepsArray.")
+
+        return IrrepsArray(f"{array.shape[-1]}x0e", array)
+
     @staticmethod
     def zeros(irreps: IntoIrreps, leading_shape, dtype=None) -> "IrrepsArray":
         r"""Create an IrrepsArray of zeros."""
@@ -609,7 +630,7 @@ class IrrepsArray:
         r"""Return the slice with respect to the multiplicities.
 
         See also:
-            :meth:`e3nn.Irreps.slice_by_mul`
+            :meth:`Irreps.slice_by_mul`
         """
         return _MulIndexSliceHelper(self)
 
@@ -618,7 +639,7 @@ class IrrepsArray:
         r"""Same as ``__getitem__`` in the irreps dimension.
 
         See also:
-            :meth:`e3nn.Irreps.slice_by_dim`
+            :meth:`Irreps.slice_by_dim`
         """
         return _DimIndexSliceHelper(self)
 
@@ -627,7 +648,7 @@ class IrrepsArray:
         r"""Return the slice with respect to the chunks.
 
         See also:
-            :meth:`e3nn.Irreps.slice_by_chunk`
+            :meth:`Irreps.slice_by_chunk`
         """
         return _ChunkIndexSliceHelper(self)
 
@@ -720,6 +741,24 @@ class IrrepsArray:
     repeat_mul_by_last_axis = axis_to_mul
     factor_mul_to_last_axis = mul_to_axis
 
+    def transform_by_log_coordinates(self, log_coordinates: jnp.ndarray, k: int = 0) -> "IrrepsArray":
+        r"""Rotate data by a rotation given by log coordinates.
+
+        Args:
+            log_coordinates (`jax.numpy.ndarray`): log coordinates
+            k (int): parity operation
+
+        Returns:
+            `IrrepsArray`: rotated data
+        """
+        log_coordinates = log_coordinates.astype(self.dtype)
+        D = {ir: ir.D_from_log_coordinates(log_coordinates, k) for ir in {ir for _, ir in self.irreps}}
+        new_list = [
+            jnp.reshape(jnp.einsum("ij,...uj->...ui", D[ir], x), self.shape[:-1] + (mul, ir.dim)) if x is not None else None
+            for (mul, ir), x in zip(self.irreps, self.list)
+        ]
+        return IrrepsArray.from_list(self.irreps, new_list, self.shape[:-1], self.dtype)
+
     def transform_by_angles(self, alpha: float, beta: float, gamma: float, k: int = 0) -> "IrrepsArray":
         r"""Rotate the data by angles according to the irreps.
 
@@ -738,7 +777,9 @@ class IrrepsArray:
             >>> x.transform_by_angles(jnp.pi, 0, 0)
             1x2e [ 0.1  0.   1.  -1.   1. ]
         """
-        # Optimization: we use only the list of arrays, not the array data
+        alpha = jnp.asarray(alpha, dtype=self.dtype)
+        beta = jnp.asarray(beta, dtype=self.dtype)
+        gamma = jnp.asarray(gamma, dtype=self.dtype)
         D = {ir: ir.D_from_angles(alpha, beta, gamma, k) for ir in {ir for _, ir in self.irreps}}
         new_list = [
             jnp.reshape(jnp.einsum("ij,...uj->...ui", D[ir], x), self.shape[:-1] + (mul, ir.dim)) if x is not None else None
@@ -756,7 +797,7 @@ class IrrepsArray:
         Returns:
             `IrrepsArray`: rotated data
         """
-        return self.transform_by_angles(*quaternion_to_angles(q), k)
+        return self.transform_by_log_coordinates(e3nn.quaternion_to_log_coordinates(q), k)
 
     def transform_by_axis_angle(self, axis: jnp.ndarray, angle: float, k: int = 0) -> "IrrepsArray":
         r"""Rotate data by a rotation given by an axis and an angle.
@@ -769,7 +810,7 @@ class IrrepsArray:
         Returns:
             `IrrepsArray`: rotated data
         """
-        return self.transform_by_angles(*axis_angle_to_angles(axis, angle), k)
+        return self.transform_by_log_coordinates(e3nn.axis_angle_to_log_coordinates(axis, angle), k)
 
     def transform_by_matrix(self, R: jnp.ndarray) -> "IrrepsArray":
         r"""Rotate data by a rotation given by a matrix.
@@ -783,7 +824,7 @@ class IrrepsArray:
         d = jnp.sign(jnp.linalg.det(R))
         R = d[..., None, None] * R
         k = (1 - d) / 2
-        return self.transform_by_angles(*matrix_to_angles(R), k)
+        return self.transform_by_angles(*e3nn.matrix_to_angles(R), k)
 
     def _convert(self, irreps: IntoIrreps) -> "IrrepsArray":
         r"""Convert the list property into an equivalent irreps.
@@ -1019,6 +1060,7 @@ def concatenate(arrays: List[IrrepsArray], axis: int = -1) -> IrrepsArray:
     if len(arrays) == 0:
         raise ValueError("Cannot concatenate empty list of IrrepsArray")
 
+    arrays = [IrrepsArray.as_irreps_array(x) for x in arrays]
     axis = _standardize_axis(axis, arrays[0].ndim)[0]
 
     jnp = _infer_backend([x.array for x in arrays])
@@ -1180,7 +1222,7 @@ def normal(
     irreps = Irreps(irreps)
 
     if normalization is None:
-        normalization = config("irrep_normalization")
+        normalization = e3nn.config("irrep_normalization")
 
     if key is None:
         warnings.warn("e3nn.normal: the key (random seed) is not provided, use the hash of the irreps as key!")
@@ -1341,7 +1383,7 @@ class _MulIndexSliceHelper:
 
     def __getitem__(self, index: slice) -> Irreps:
         if not isinstance(index, slice):
-            raise IndexError("IrrepsArray.slice_by_mul only supports slices.")
+            raise IndexError("IrrepsArray.slice_by_mul only supports one slices (like IrrepsArray.slice_by_mul[2:4]).")
         start, stop, stride = index.indices(self.irreps_array.irreps.num_irreps)
         if stride != 1:
             raise NotImplementedError("IrrepsArray.slice_by_mul does not support strides.")
@@ -1369,7 +1411,7 @@ class _DimIndexSliceHelper:
 
     def __getitem__(self, index: slice) -> Irreps:
         if not isinstance(index, slice):
-            raise IndexError("IrrepsArray.slice_by_dim only supports slices.")
+            raise IndexError("IrrepsArray.slice_by_dim only supports slices (like IrrepsArray.slice_by_dim[2:4]).")
         return self.irreps_array[..., index]
 
 
@@ -1381,7 +1423,7 @@ class _ChunkIndexSliceHelper:
 
     def __getitem__(self, index: slice) -> Irreps:
         if not isinstance(index, slice):
-            raise IndexError("IrrepsArray.slice_by_chunk only supports slices.")
+            raise IndexError("IrrepsArray.slice_by_chunk only supports slices (like IrrepsArray.slice_by_chunk[2:4]).")
         start, stop, stride = index.indices(len(self.irreps_array.irreps))
 
         return IrrepsArray.from_list(
