@@ -3,9 +3,11 @@ History of the different versions of the code:
 - Initially developed by Mario Geiger in `e3nn`
 - Ported in julia by Song Kim https://github.com/songk42/ReducedTensorProduct.jl
 - Ported in `e3nn-jax` by Mario Geiger
+- Optimized the symmetric case by Ameya Daigavane and Mario Geiger
 """
 import functools
 import itertools
+from math import prod
 from typing import FrozenSet, Iterator, List, Optional, Sequence, Tuple, Union
 
 import numpy as np
@@ -13,7 +15,6 @@ import numpy as np
 import e3nn_jax as e3nn
 from e3nn_jax import perm
 from e3nn_jax._src.util.math_numpy import basis_intersection, round_to_sqrt_rational
-from e3nn_jax._src.util.prod import prod
 
 
 def reduced_tensor_product_basis(
@@ -190,8 +191,10 @@ def _reduced_tensor_product_basis(
         and len(irreps_tuple) > 1
         and irreps_tuple[0].num_irreps > 1
     ):
-        return _optimized_reduced_symmetric_tensor_product_basis(
-            irreps_tuple[0], len(irreps_tuple), epsilon=epsilon, keep_ir=keep_ir
+        return _rounding(
+            _optimized_reduced_symmetric_tensor_product_basis(
+                irreps_tuple[0], len(irreps_tuple), epsilon=epsilon, keep_ir=keep_ir
+            )
         )
 
     # General case
@@ -223,12 +226,12 @@ def _reduced_tensor_product_basis(
             (fa, a) = bases[0]
             (fb, b) = bases[1]
             f = frozenset(fa | fb)
-            ab = reduce_basis_product(a, b, keep_ir, round_fn=round_to_sqrt_rational)
+            ab = reduce_basis_product(a, b, keep_ir)
             if len(subrepr_permutation(f, perm_repr)) == 1:
                 return ab.regroup()
             p = reduce_subgroup_permutation(f, perm_repr, dims)
-            ab = constrain_rotation_basis_by_permutation_basis(ab, p, epsilon=epsilon, round_fn=round_to_sqrt_rational)
-            return ab.regroup()
+            ab = constrain_rotation_basis_by_permutation_basis(ab, p, epsilon=epsilon)
+            return _rounding(ab.regroup())
 
         # greedy algorithm
         min_p = np.inf
@@ -396,7 +399,7 @@ def _optimized_reduced_symmetric_tensor_product_basis(
         # Compute basis product, two terms at a time.
         current_term = non_zero_terms_reshaped[0]
         for next_term in non_zero_terms_reshaped[1:]:
-            current_term = reduce_basis_product(current_term, next_term, keep_ir, round_fn=round_to_sqrt_rational)
+            current_term = reduce_basis_product(current_term, next_term)
         product_basis = current_term
 
         sum_of_permuted_bases = None
@@ -621,3 +624,9 @@ def reduce_permutation_matrix(
     np.testing.assert_allclose(Q @ Q.T, np.eye(d_sym))
 
     return Q.reshape(d_sym, *dims)
+
+
+def _rounding(x: e3nn.IrrepsArray) -> e3nn.IrrepsArray:
+    # print(round_to_sqrt_rational(1/2 + 1e-13, 2**20) == 0.5)  # True
+    # print(round_to_sqrt_rational(1/2 + 1e-12, 2**20) == 0.5)  # False
+    return e3nn.IrrepsArray(x.irreps, round_to_sqrt_rational(x.array, 2**20))
