@@ -4,7 +4,7 @@ import jax.numpy as jnp
 # matrix
 
 
-def rand_matrix(key, shape, dtype=jnp.float32):
+def rand_matrix(key, shape=(), dtype=jnp.float32):
     r"""Random rotation matrix.
 
     Args:
@@ -17,10 +17,23 @@ def rand_matrix(key, shape, dtype=jnp.float32):
     return angles_to_matrix(*rand_angles(key, shape, dtype=dtype))
 
 
+def rotation_angle_from_matrix(R):
+    r"""Angle of rotation from a rotation matrix.
+
+    Args:
+        m (`jax.numpy.ndarray`): array of shape :math:`(..., 3, 3)`
+
+    Returns:
+        `jax.numpy.ndarray`: array of shape :math:`(...)`
+    """
+    trace = R[..., 0, 0] + R[..., 1, 1] + R[..., 2, 2]
+    return jnp.arccos(jnp.clip((trace - 1.0) / 2.0, -1.0, 1.0))
+
+
 # angles
 
 
-def identity_angles(shape, dtype=jnp.float32):
+def identity_angles(shape=(), dtype=jnp.float32):
     r"""Angles of the identity rotation.
 
     Args:
@@ -34,7 +47,7 @@ def identity_angles(shape, dtype=jnp.float32):
     return jnp.zeros(shape, dtype), jnp.zeros(shape, dtype), jnp.zeros(shape, dtype)
 
 
-def rand_angles(key, shape, dtype=jnp.float32):
+def rand_angles(key, shape=(), dtype=jnp.float32):
     r"""Random rotation angles.
 
     Args:
@@ -89,10 +102,30 @@ def inverse_angles(a, b, c):
     return -c, -b, -a
 
 
+def rotation_angle_from_angles(a1, b1, c1, a2, b2, c2):
+    r"""Angle of rotation from two triplets of angles.
+
+    Args:
+        alpha1 (`jax.numpy.ndarray`): array of shape :math:`(...)`
+        beta1 (`jax.numpy.ndarray`): array of shape :math:`(...)`
+        gamma1 (`jax.numpy.ndarray`): array of shape :math:`(...)`
+        alpha2 (`jax.numpy.ndarray`): array of shape :math:`(...)`
+        beta2 (`jax.numpy.ndarray`): array of shape :math:`(...)`
+        gamma2 (`jax.numpy.ndarray`): array of shape :math:`(...)`
+
+    Returns:
+        `jax.numpy.ndarray`: array of shape :math:`(...)`
+    """
+    R1 = angles_to_matrix(a1, b1, c1)
+    R2 = angles_to_matrix(a2, b2, c2)
+    R1, R2 = jnp.broadcast_arrays(R1, R2)
+    return rotation_angle_from_matrix(R1 @ R2.T)
+
+
 # quaternions
 
 
-def identity_quaternion(shape, dtype=jnp.float32):
+def identity_quaternion(shape=(), dtype=jnp.float32):
     r"""Quaternion of identity rotation.
 
     Args:
@@ -105,7 +138,7 @@ def identity_quaternion(shape, dtype=jnp.float32):
     return q.at[..., 0].set(1)  # or -1...
 
 
-def rand_quaternion(key, shape, dtype=jnp.float32):
+def rand_quaternion(key, shape=(), dtype=jnp.float32):
     r"""Generate random quaternion.
 
     Args:
@@ -154,10 +187,26 @@ def inverse_quaternion(q):
     return q.at[..., 1:].multiply(-1)
 
 
+def rotation_angle_from_quaternion(q1, q2):
+    r"""Rotation angle between two quaternions.
+
+    Args:
+        q1 (`jax.numpy.ndarray`): array of shape :math:`(..., 4)`
+        q2 (`jax.numpy.ndarray`): array of shape :math:`(..., 4)`
+
+    Returns:
+        `jax.numpy.ndarray`: array of shape :math:`(...)`
+    """
+    q1, q2 = jnp.broadcast_arrays(q1, q2)
+    dot = jnp.sum(q1 * q2, axis=-1)
+    cap = jnp.minimum(jnp.abs(dot), 1.0)
+    return 2.0 * jnp.arccos(cap)
+
+
 # axis-angle
 
 
-def rand_axis_angle(key, shape, dtype=jnp.float32):
+def rand_axis_angle(key, shape=(), dtype=jnp.float32):
     r"""Generate random rotation as axis-angle.
 
     Args:
@@ -189,10 +238,28 @@ def compose_axis_angle(axis1, angle1, axis2, angle2):
     )
 
 
+def rotation_angle_from_axis_angle(axis1, angle1, axis2, angle2):
+    r"""Rotation angle between :math:`(\vec x_1, \alpha_1)` and :math:`(\vec x_2, \alpha_2)`.
+
+    Args:
+        axis1 (`jax.numpy.ndarray`): array of shape :math:`(..., 3)`
+        angle1 (`jax.numpy.ndarray`): array of shape :math:`(...)`
+        axis2 (`jax.numpy.ndarray`): array of shape :math:`(..., 3)`
+        angle2 (`jax.numpy.ndarray`): array of shape :math:`(...)`
+
+    Returns:
+        `jax.numpy.ndarray`: array of shape :math:`(...)`
+    """
+    return rotation_angle_from_quaternion(
+        axis_angle_to_quaternion(axis1, angle1),
+        axis_angle_to_quaternion(axis2, angle2),
+    )
+
+
 # log coordinates
 
 
-def identity_log_coordinates(shape, dtype=jnp.float32):
+def identity_log_coordinates(shape=(), dtype=jnp.float32):
     r"""Log coordinates of identity rotation.
 
     Args:
@@ -204,7 +271,7 @@ def identity_log_coordinates(shape, dtype=jnp.float32):
     return jnp.zeros(shape + (3,), dtype=dtype)
 
 
-def rand_log_coordinates(key, shape, dtype=jnp.float32):
+def rand_log_coordinates(key, shape=(), dtype=jnp.float32):
     r"""Generate random rotation as log coordinates.
 
     Args:
@@ -244,6 +311,22 @@ def inverse_log_coordinates(log):
     return -log
 
 
+def rotation_angle_from_log_coordinates(log1, log2):
+    r"""Rotation angle between a pair of log coordinates.
+
+    Args:
+        log1 (`jax.numpy.ndarray`): array of shape :math:`(..., 3)`
+        log2 (`jax.numpy.ndarray`): array of shape :math:`(..., 3)`
+
+    Returns:
+        `jax.numpy.ndarray`: array of shape :math:`(...)`
+    """
+    return rotation_angle_from_quaternion(
+        log_coordinates_to_quaternion(log1),
+        log_coordinates_to_quaternion(log2),
+    )
+
+
 # conversions
 
 
@@ -326,6 +409,7 @@ def angles_to_matrix(alpha, beta, gamma):
 
 def matrix_to_angles(R):
     r"""Conversion from matrix to angles.
+    Warning: this function is not differentiable at rotation angles :math:`\pi`.
 
     Args:
         R (`jax.numpy.ndarray`): array of shape :math:`(..., 3, 3)`
@@ -406,12 +490,14 @@ def quaternion_to_axis_angle(q):
 
 
 def _normalize(x):
-    n = jnp.linalg.norm(x, axis=-1, keepdims=True)
-    return x / jnp.where(n > 0, n, 1.0)
+    n2 = jnp.sum(x**2, axis=-1, keepdims=True)
+    n2 = jnp.where(n2 > 0.0, n2, 1.0)
+    return x / jnp.sqrt(n2)
 
 
 def matrix_to_axis_angle(R):
     r"""Conversion from matrix to axis-angle.
+    Warning: this function is not differentiable at rotation angles :math:`\pi`.
 
     Args:
         R (`jax.numpy.ndarray`): array of shape :math:`(..., 3, 3)`
@@ -421,8 +507,7 @@ def matrix_to_axis_angle(R):
         angle (`jax.numpy.ndarray`): array of shape :math:`(...)`
     """
     # assert jnp.allclose(jnp.linalg.det(R), 1)
-    tr = R[..., 0, 0] + R[..., 1, 1] + R[..., 2, 2]
-    angle = jnp.arccos(jnp.clip((tr - 1) / 2, -1, 1))
+    angle = rotation_angle_from_matrix(R)
     axis = jnp.stack(
         [
             R[..., 2, 1] - R[..., 1, 2],
@@ -543,8 +628,10 @@ def log_coordinates_to_axis_angle(log_coordinates):
         axis (`jax.numpy.ndarray`): array of shape :math:`(..., 3)`
         angle (`jax.numpy.ndarray`): array of shape :math:`(...)`
     """
-    axis = _normalize(log_coordinates)
-    angle = jnp.linalg.norm(log_coordinates, axis=-1)
+    n2 = jnp.sum(log_coordinates**2, axis=-1)
+    n2_ = jnp.where(n2 > 0.0, n2, 1.0)
+    axis = log_coordinates / jnp.sqrt(n2_)[..., None]
+    angle = jnp.where(n2 > 0.0, jnp.sqrt(n2_), 0.0)
     return axis, angle
 
 
@@ -651,7 +738,7 @@ def angles_to_xyz(alpha, beta):
 
 
 def xyz_to_angles(xyz):
-    r"""Convert a point :math:`\vec r = (x, y, z)` on the sphere into angles :math:`(\alpha, \beta)`.
+    r"""The rotation :math:`R(\alpha, \beta, 0)` such that :math:`\vec r = R \vec e_y`.
 
     .. math::
 
