@@ -55,7 +55,7 @@ def tensor_product(
            0.    -0.66   4.08   0.     0.     0.     0.     0.     1.     2.
            3.     4.     5.     9.9   10.97   9.27  -1.97  12.34  15.59  12.73]
 
-        Usage in combination with `Linear`:
+        Usage in combination with `haiku.Linear` or `flax.Linear`:
 
         >>> import flax.linen as nn
         >>> linear = e3nn.flax.Linear("3x1e")
@@ -74,6 +74,10 @@ def tensor_product(
         input2 = input2.regroup()
 
     if filter_ir_out is not None:
+        if isinstance(filter_ir_out, str):
+            filter_ir_out = Irreps(filter_ir_out)
+        if isinstance(filter_ir_out, Irrep):
+            filter_ir_out = [filter_ir_out]
         filter_ir_out = [Irrep(ir) for ir in filter_ir_out]
 
     irreps_out = []
@@ -81,7 +85,6 @@ def tensor_product(
     for i_1, (mul_1, ir_1) in enumerate(input1.irreps):
         for i_2, (mul_2, ir_2) in enumerate(input2.irreps):
             for ir_out in ir_1 * ir_2:
-
                 if filter_ir_out is not None and ir_out not in filter_ir_out:
                     continue
 
@@ -166,7 +169,6 @@ def elementwise_tensor_product(
     for i, ((mul, ir_1), (mul_2, ir_2)) in enumerate(zip(irreps_in1, irreps_in2)):
         assert mul == mul_2
         for ir in ir_1 * ir_2:
-
             if filter_ir_out is not None and ir not in filter_ir_out:
                 continue
 
@@ -233,7 +235,6 @@ def tensor_square(
     for i_1, (mul_1, ir_1) in enumerate(input.irreps):
         for i_2, (mul_2, ir_2) in enumerate(input.irreps):
             for ir_out in ir_1 * ir_2:
-
                 if normalized_input:
                     if irrep_normalization == "component":
                         alpha = ir_1.dim * ir_2.dim * ir_out.dim
